package util

import (
	"sync"
	"context"
	"amuz.es/src/infra/cpu_ctrl/logger"
)

var (
	log = logger.NewLogger("util")
)

type handler struct {
	errorChan chan error
	ctx       context.Context
	canceler  context.CancelFunc
	waiter    *sync.WaitGroup
}

type Handler interface {
	NotifyError(err error)
	Error() <-chan error
	Done() <-chan struct{}
	GracefullWait()
	IncreaseWait()
	DecreaseWait()
}

func NewHandler() Handler {
	ctx, canceler := context.WithCancel(context.Background())
	return &handler{
		ctx:       ctx,
		canceler:  canceler,
		waiter:    &sync.WaitGroup{},
		errorChan: make(chan error,1),
	}
}

func (h *handler) NotifyError(err error) { h.errorChan <- err }
func (h *handler) Error() <-chan error   { return h.errorChan }
func (h *handler) Done() <-chan struct{} { return h.ctx.Done() }
func (h *handler) GracefullWait() {
	if h.ctx.Err() == nil {
		h.canceler()
	}
	h.waiter.Wait()
	close(h.errorChan)

	for remainError := range h.errorChan {
		log.Errorf("%s", remainError)
	}
}
func (h *handler) IncreaseWait() { h.waiter.Add(1) }
func (h *handler) DecreaseWait() { h.waiter.Done() }
